@@ -21,17 +21,51 @@ import pyutilib.misc
 import pyutilib.component.core
 import string
 import re
-
 import xml.dom.minidom
-
 import time
+import logging
 
 
-class CPLEX(ILMLicensedSystemCallSolver):
+class CPLEX(OptSolver):
     """The CPLEX LP/MIP solver
     """
 
-    pyutilib.component.core.alias('cplex', doc='Shell interface to the CPLEX LP/MIP solver')
+    pyutilib.component.core.alias('cplex', doc='The CPLEX MIP solver')
+
+    def __new__(cls, *args, **kwds):
+        try:
+            mode = kwds['solver_io']
+            if mode is None:
+                mode = 'lp'
+            del kwds['solver_io']
+        except KeyError:
+            mode = 'lp'
+        #
+        if mode  == 'lp':
+            return SolverFactory('_cplex_shell', **kwds)
+        if mode == 'python':
+            opt = SolverFactory('_cplex_direct', **kwds)
+            if opt is None:
+                logging.getLogger('coopr.plugins').error('Python API for CPLEX is not installed')
+                return
+            return opt
+        #
+        if mode == 'os':
+            opt = SolverFactory('_ossolver', **kwds)
+        elif mode == 'nl':
+            opt = SolverFactory('_asl', **kwds)
+        else:
+            logging.getLogger('coopr.plugins').error('Unknown IO type: %s' % mode)
+            return
+        opt.set_options('solver=cplexamp')
+        return opt
+
+
+class CPLEXSHELL(ILMLicensedSystemCallSolver):
+    """Shell interface to the CPLEX LP/MIP solver
+    """
+
+    pyutilib.component.core.alias('_cplex_shell', doc='Shell interface to the CPLEX LP/MIP solver')
 
     def __init__(self, **kwds):
         #
@@ -74,105 +108,105 @@ class CPLEX(ILMLicensedSystemCallSolver):
     #
     def _hasIntegerVariables(self, instance):
 
-       import coopr.pyomo.base.var
-       from coopr.pyomo.base import Var
-       from coopr.pyomo.base.set_types import IntegerSet, BooleanSet
+        import coopr.pyomo.base.var
+        from coopr.pyomo.base import Var
+        from coopr.pyomo.base.set_types import IntegerSet, BooleanSet
 
-       for variable in instance.active_components(Var).values():
+        for variable in instance.active_components(Var).values():
 
-           if (isinstance(variable.domain, IntegerSet) is True) or (isinstance(variable.domain, BooleanSet) is True):
+            if (isinstance(variable.domain, IntegerSet) is True) or (isinstance(variable.domain, BooleanSet) is True):
 
-               return True
+                return True
 
-       return False
+        return False
 
     #
     # CPLEX has a simple, easy-to-use warm-start capability.
     #
     def warm_start_capable(self):
-       return True
+        return True
 
     #
     # write a warm-start file in the CPLEX MST format.
     #
     def warm_start(self, instance):
 
-       import coopr.pyomo.base.var
-       from coopr.pyomo.base import Var
+        import coopr.pyomo.base.var
+        from coopr.pyomo.base import Var
 
-       self.warm_start_file_name = pyutilib.services.TempfileManager.create_tempfile(suffix = '.cplex.mst')
+        self.warm_start_file_name = pyutilib.services.TempfileManager.create_tempfile(suffix = '.cplex.mst')
 
-       doc = xml.dom.minidom.Document()
-       root_element = doc.createElement("CPLEXSolution")
-       root_element.setAttribute("version","1.0")
-       doc.appendChild(root_element)
+        doc = xml.dom.minidom.Document()
+        root_element = doc.createElement("CPLEXSolution")
+        root_element.setAttribute("version","1.0")
+        doc.appendChild(root_element)
 
-       # currently not populated.
-       header_element = doc.createElement("header")
-       # currently not populated.
-       quality_element = doc.createElement("quality")
-       # definitely populated!
-       variables_element = doc.createElement("variables")
+        # currently not populated.
+        header_element = doc.createElement("header")
+        # currently not populated.
+        quality_element = doc.createElement("quality")
+        # definitely populated!
+        variables_element = doc.createElement("variables")
 
-       root_element.appendChild(header_element)
-       root_element.appendChild(quality_element)
-       root_element.appendChild(variables_element)
+        root_element.appendChild(header_element)
+        root_element.appendChild(quality_element)
+        root_element.appendChild(variables_element)
 
-       # for each variable, add a child to the variables element.
-       # both continuous and discrete are accepted (and required,
-       # depending on other options), according to the CPLEX manual.
-       output_index = 0
-       for variable in instance.active_components(Var).values():
+        # for each variable, add a child to the variables element.
+        # both continuous and discrete are accepted (and required,
+        # depending on other options), according to the CPLEX manual.
+        output_index = 0
+        for variable in instance.active_components(Var).values():
 
-           for index in variable._varval.keys():
+            for index in variable._varval.keys():
 
-               if (variable[index].status != coopr.pyomo.base.var.VarStatus.unused) and (variable[index].value != None) and (variable[index].fixed == False):
+                if (variable[index].status != coopr.pyomo.base.var.VarStatus.unused) and (variable[index].value != None) and (variable[index].fixed == False):
 
-                   variable_element = doc.createElement("variable")
-                   name = variable[index].label
-                   name = name.replace('[','(')
-                   name = name.replace(']',')')
-                   variable_element.setAttribute("name", name)
-                   variable_element.setAttribute("index", str(output_index))
-                   variable_element.setAttribute("value", str(variable[index].value))
+                    variable_element = doc.createElement("variable")
+                    name = variable[index].label
+                    name = name.replace('[','(')
+                    name = name.replace(']',')')
+                    variable_element.setAttribute("name", name)
+                    variable_element.setAttribute("index", str(output_index))
+                    variable_element.setAttribute("value", str(variable[index].value))
 
-                   variables_element.appendChild(variable_element)
+                    variables_element.appendChild(variable_element)
 
-                   output_index = output_index + 1
+                    output_index = output_index + 1
 
-       mst_file = open(self.warm_start_file_name,'w')
-       doc.writexml(mst_file, indent="    ", newl="\n")
-       mst_file.close()
+        mst_file = open(self.warm_start_file_name,'w')
+        doc.writexml(mst_file, indent="    ", newl="\n")
+        mst_file.close()
 
     # over-ride presolve to extract the warm-start keyword, if specified.
     def _presolve(self, *args, **kwds):
 
-       # if the first argument is a string (representing a filename),
-       # then we don't have an instance => the solver is being applied
-       # to a file.
+        # if the first argument is a string (representing a filename),
+        # then we don't have an instance => the solver is being applied
+        # to a file.
 
-       self.warm_start_solve = kwds.pop( 'warmstart', False )
+        self.warm_start_solve = kwds.pop( 'warmstart', False )
 
-       if (len(args) > 0) and (isinstance(args[0],basestring) is False):
+        if (len(args) > 0) and (isinstance(args[0],basestring) is False):
 
-          # write the warm-start file - currently only supports MIPs.
-          # we only know how to deal with a single problem instance.
-          if self.warm_start_solve is True:
+            # write the warm-start file - currently only supports MIPs.
+            # we only know how to deal with a single problem instance.
+            if self.warm_start_solve is True:
 
-             if len(args) != 1:
-                msg = "CPLEX _presolve method can only handle a single "      \
-                      "problem instance - %s were supplied"
-                raise ValueError, msg % len(args)
+                if len(args) != 1:
+                    msg = "CPLEX _presolve method can only handle a single "      \
+                          "problem instance - %s were supplied"
+                    raise ValueError, msg % len(args)
 
-             if self._hasIntegerVariables(args[0]) is True:
-                start_time = time.time()
-                self.warm_start(args[0])
-                end_time = time.time()
-                if self._report_timing is True:
-                   print "Warm start write time="+str(end_time-start_time)+" seconds"
+                if self._hasIntegerVariables(args[0]) is True:
+                    start_time = time.time()
+                    self.warm_start(args[0])
+                    end_time = time.time()
+                    if self._report_timing is True:
+                        print "Warm start write time="+str(end_time-start_time)+" seconds"
 
-       # let the base class handle any remaining keywords/actions.
-       ILMLicensedSystemCallSolver._presolve(self, *args, **kwds)
+        # let the base class handle any remaining keywords/actions.
+        ILMLicensedSystemCallSolver._presolve(self, *args, **kwds)
 
     def executable(self):
         executable = pyutilib.services.registered_executable("cplex")
@@ -199,9 +233,9 @@ class CPLEX(ILMLicensedSystemCallSolver):
         # Define results file
         #
         if self._results_format is None or self._results_format == ResultsFormat.soln:
-           self.results_file = self.soln_file
+            self.results_file = self.soln_file
         elif self._results_format == ResultsFormat.sol:
-           self.results_file = self.sol_file
+            self.results_file = self.sol_file
 
         #
         # Write the CPLEX execution script
@@ -214,13 +248,13 @@ class CPLEX(ILMLicensedSystemCallSolver):
         if (self.options.mipgap is not None) and (self.options.mipgap > 0.0):
             cplex_script_file.write("set mip tolerances mipgap "+`self.options.mipgap`+"\n")
         for key in self.options:
-                if key == 'relax_integrality' or key == 'mipgap':
-                    continue
-                elif isinstance(self.options[key],basestring) and ' ' in self.options[key]:
-                    opt = " ".join(key.split('_'))+" "+str(self.options[key])
-                else:
-                    opt = " ".join(key.split('_'))+" "+str(self.options[key])
-                cplex_script_file.write("set "+opt+"\n")
+            if key == 'relax_integrality' or key == 'mipgap':
+                continue
+            elif isinstance(self.options[key],basestring) and ' ' in self.options[key]:
+                opt = " ".join(key.split('_'))+" "+str(self.options[key])
+            else:
+                opt = " ".join(key.split('_'))+" "+str(self.options[key])
+            cplex_script_file.write("set "+opt+"\n")
         cplex_script_file.write("read "+problem_files[0]+"\n")
 
         # if we're dealing with an LP, the MST file will be empty.
@@ -239,15 +273,15 @@ class CPLEX(ILMLicensedSystemCallSolver):
         # dump the script and warm-start file names for the
         # user if we're keeping files around.
         if self.keepFiles:
-           print "Solver script file=" + self.cplex_script_file_name
-           if (self.warm_start_solve is True) and (self.warm_start_file_name is not None):
-              print "Solver warm-start file=" + self.warm_start_file_name
+            print "Solver script file=" + self.cplex_script_file_name
+            if (self.warm_start_solve is True) and (self.warm_start_file_name is not None):
+                print "Solver warm-start file=" + self.warm_start_file_name
 
         #
         # Define command line
         #
         if self._problem_format in [ProblemFormat.cpxlp, ProblemFormat.mps]:
-           proc = self._timer + " " + self.executable() + " < " + self.cplex_script_file_name
+            proc = self._timer + " " + self.executable() + " < " + self.cplex_script_file_name
         return pyutilib.misc.Bunch(cmd=proc, log_file=self.log_file, env=None)
 
     def process_logfile(self):
@@ -274,9 +308,9 @@ class CPLEX(ILMLicensedSystemCallSolver):
         for line in output.split("\n"):
             tokens = re.split('[ \t]+',line.strip())
             if len(tokens) > 3 and tokens[0] == "CPLEX" and tokens[1] == "Error":
-                # IMPT: See below - cplex can generate an error line and then terminate fine, e.g., in CPLEX 12.1.
-                #       To handle these cases, we should be specifying some kind of termination criterion always
-                #       in the course of parsing a log file (we aren't doing so currently - just in some conditions).
+            # IMPT: See below - cplex can generate an error line and then terminate fine, e.g., in CPLEX 12.1.
+            #       To handle these cases, we should be specifying some kind of termination criterion always
+            #       in the course of parsing a log file (we aren't doing so currently - just in some conditions).
                 results.solver.status=SolverStatus.error
                 results.solver.error = " ".join(tokens)
             elif len(tokens) >= 3 and tokens[0] == "ILOG" and tokens[1] == "CPLEX":
@@ -300,9 +334,9 @@ class CPLEX(ILMLicensedSystemCallSolver):
             elif len(tokens) >= 5 and tokens[4] == "MAXIMIZE":
                 results.problem.sense = ProblemSense.maximize
             elif len(tokens) >= 4 and tokens[0] == "Solution" and tokens[1] == "time" and tokens[2] == "=":
-               # technically, I'm not sure if this is CPLEX user time or user+system - CPLEX doesn't appear
-               # to differentiate, and I'm not sure we can always provide a break-down.
-               results.solver.user_time = eval(tokens[3])
+                # technically, I'm not sure if this is CPLEX user time or user+system - CPLEX doesn't appear
+                # to differentiate, and I'm not sure we can always provide a break-down.
+                results.solver.user_time = eval(tokens[3])
             elif len(tokens) >= 4 and tokens[0] == "Dual" and tokens[1] == "simplex" and tokens[3] == "Optimal:":
                 results.solver.termination_condition = TerminationCondition.optimal
                 results.solver.termination_message = ' '.join(tokens)
@@ -310,9 +344,9 @@ class CPLEX(ILMLicensedSystemCallSolver):
                 # if CPLEX has previously printed an error message, reduce it to a warning -
                 # there is a strong indication it recovered, but we can't be sure.
                 if results.solver.status == SolverStatus.error:
-                   results.solver.status = SolverStatus.warning
+                    results.solver.status = SolverStatus.warning
                 else:
-                   results.solver.status = SolverStatus.ok
+                    results.solver.status = SolverStatus.ok
                 results.solver.termination_condition = TerminationCondition.infeasible
                 results.solver.termination_message = ' '.join(tokens)
             # for the case below, CPLEX sometimes reports "true" optimal (the first case)
@@ -322,27 +356,27 @@ class CPLEX(ILMLicensedSystemCallSolver):
                 # if CPLEX has previously printed an error message, reduce it to a warning -
                 # there is a strong indication it recovered, but we can't be sure.
                 if results.solver.status == SolverStatus.error:
-                   results.solver.status = SolverStatus.warning
+                    results.solver.status = SolverStatus.warning
                 else:
-                   results.solver.status = SolverStatus.ok
+                    results.solver.status = SolverStatus.ok
                 results.solver.termination_condition = TerminationCondition.optimal
                 results.solver.termination_message = ' '.join(tokens)
             elif len(tokens) >= 3 and tokens[0] == "Presolve" and tokens[2] == "Infeasible.":
                 # if CPLEX has previously printed an error message, reduce it to a warning -
                 # there is a strong indication it recovered, but we can't be sure.
                 if results.solver.status == SolverStatus.error:
-                   results.solver.status = SolverStatus.warning
+                    results.solver.status = SolverStatus.warning
                 else:
-                   results.solver.status = SolverStatus.ok
+                    results.solver.status = SolverStatus.ok
                 results.solver.termination_condition = TerminationCondition.infeasible
                 results.solver.termination_message = ' '.join(tokens)
             elif len(tokens) >= 5 and tokens[0] == "Presolve" and tokens[2] == "Unbounded" and tokens[4] == "infeasible.":
                 # if CPLEX has previously printed an error message, reduce it to a warning -
                 # there is a strong indication it recovered, but we can't be sure.
                 if results.solver.status == SolverStatus.error:
-                   results.solver.status = SolverStatus.warning
+                    results.solver.status = SolverStatus.warning
                 else:
-                   results.solver.status = SolverStatus.ok
+                    results.solver.status = SolverStatus.ok
                 # It isn't clear whether we can determine if the problem is unbounded from
                 # CPLEX's output.
                 results.solver.termination_condition = TerminationCondition.unbounded
@@ -365,25 +399,26 @@ class CPLEX(ILMLicensedSystemCallSolver):
         extract_slacks = False
         extract_reduced_costs = False
         for suffix in self.suffixes:
-           flag=False
-           if re.match(suffix,"dual"):
+            flag=False
+            if re.match(suffix,"dual"):
                 extract_duals = True
                 flag=True
-           if re.match(suffix,"slack"):
+            if re.match(suffix,"slack"):
                 extract_slacks = True
                 flag=True
-           if re.match(suffix,"rc"):
+            if re.match(suffix,"rc"):
                 extract_reduced_costs = True
                 flag=True
-           if not flag:
-              raise RuntimeError,"***The CPLEX solver plugin cannot extract solution suffix="+suffix
+            if not flag:
+                raise RuntimeError,"***The CPLEX solver plugin cannot extract solution suffix="+suffix
 
         lp_solution = False
         if not os.path.exists(self.soln_file):
-           return
+            return
 
         soln = Solution()
         soln.objective['__default_objective__'].value=None
+        soln_variable = soln.variable # caching for efficiency
         INPUT = open(self.soln_file,"r")
         results.problem.number_of_objectives=1
         mip_problem=False
@@ -399,37 +434,32 @@ class CPLEX(ILMLicensedSystemCallSolver):
                 variable_reduced_cost = None
                 variable_status = None
                 for i in range(1,len(tokens)):
-                   field_name =  string.strip(tokens[i].split('=')[0])
-                   field_value = (string.strip(tokens[i].split('=')[1])).lstrip("\"").rstrip("\"")
-                   if field_name == "name":
-                      variable_name = field_value
-                   elif field_name == "value":
-                      variable_value = field_value
-                   elif (extract_reduced_costs is True) and (field_name == "reducedCost"):
-                      variable_reduced_cost = field_value
-                   elif (extract_reduced_costs is True) and (field_name == "status"):
-                      variable_status = field_value
+                    field_name =  string.strip(tokens[i].split('=')[0])
+                    field_value = (string.strip(tokens[i].split('=')[1])).lstrip("\"").rstrip("\"")
+                    if field_name == "name":
+                        variable_name = field_value
+                    elif field_name == "value":
+                        variable_value = field_value
+                    elif (extract_reduced_costs is True) and (field_name == "reducedCost"):
+                        variable_reduced_cost = field_value
+                    elif (extract_reduced_costs is True) and (field_name == "status"):
+                        variable_status = field_value
 
                 # skip the "constant-one" variable, used to capture/retain objective offsets in the CPLEX LP format.
                 if variable_name != "ONE_VAR_CONSTANT":
-                   variable = None # cache the solution variable reference, as the getattr is expensive.
-                   try:
-                       variable = soln.variable[variable_name]
-                       variable.value = eval(variable_value)
-                   except:
-                       variable.value = variable_value
-                   if (variable_reduced_cost is not None) and (extract_reduced_costs is True):
+                    variable = soln_variable[variable_name] = {"Value" : eval(variable_value), "Id" : len(soln_variable)}
+                    if (variable_reduced_cost is not None) and (extract_reduced_costs is True):
                         try:
-                            variable.rc = eval(variable_reduced_cost)
+                            variable["Rc"] = eval(variable_reduced_cost)
                             if variable_status is not None:
-                               if variable_status == "LL":
-                                  variable.lrc = eval(variable_reduced_cost)
-                               else:
-                                  variable.lrc = 0.0
-                               if variable_status == "UL":
-                                  variable.urc = eval(variable_reduced_cost)
-                               else:
-                                  variable.urc = 0.0
+                                if variable_status == "LL":
+                                    variable["Lrc"] = eval(variable_reduced_cost)
+                                else:
+                                    variable["Lrc"] = 0.0
+                                if variable_status == "UL":
+                                    variable["Urc"] = eval(variable_reduced_cost)
+                                else:
+                                    variable["Urc"] = 0.0
                         except:
                             raise ValueError, "Unexpected reduced-cost value="+str(variable_reduced_cost)+" encountered for variable="+variable_name
             elif (tokens[0] == "constraint") and ((extract_duals is True) or (extract_slacks is True)):
@@ -437,19 +467,19 @@ class CPLEX(ILMLicensedSystemCallSolver):
                 constraint_dual = None
                 constaint = None # cache the solution constraint reference, as the getattr is expensive.
                 for i in range(1,len(tokens)):
-                   field_name =  string.strip(tokens[i].split('=')[0])
-                   field_value = (string.strip(tokens[i].split('=')[1])).lstrip("\"").rstrip("\"")
-                   if field_name == "name":
-                      constraint_name = field_value
-                      constraint = soln.constraint[constraint_name]
-                   elif (extract_duals is True) and (field_name == "dual"): # for LPs
-                      # assumes the name field is first.
-                      if eval(field_value) != 0.0:
-                        constraint.dual = eval(field_value)
-                   elif (extract_slacks is True) and (field_name == "slack"): # for MIPs
-                      # assumes the name field is first.
-                      if eval(field_value) != 0.0:
-                        constraint.slack = eval(field_value)
+                    field_name =  string.strip(tokens[i].split('=')[0])
+                    field_value = (string.strip(tokens[i].split('=')[1])).lstrip("\"").rstrip("\"")
+                    if field_name == "name":
+                        constraint_name = field_value
+                        constraint = soln.constraint[constraint_name]
+                    elif (extract_duals is True) and (field_name == "dual"): # for LPs
+                        # assumes the name field is first.
+                        if eval(field_value) != 0.0:
+                            constraint.dual = eval(field_value)
+                    elif (extract_slacks is True) and (field_name == "slack"): # for MIPs
+                        # assumes the name field is first.
+                        if eval(field_value) != 0.0:
+                            constraint.slack = eval(field_value)
             elif tokens[0].startswith("problemName"):
                 filename = (string.strip(tokens[0].split('=')[1])).lstrip("\"").rstrip("\"")
                 #print "HERE",filename
@@ -513,26 +543,26 @@ class CPLEX(ILMLicensedSystemCallSolver):
         # accept regular expression directly.
         filename_list = os.listdir(".")
         for filename in filename_list:
-           # CPLEX temporary files come in two flavors - cplex.log and clone*.log.
-           # the latter is the case for multi-processor environments.
-           # IMPT: trap the possible exception raised by the file not existing.
-           #       this can occur in pyro environments where > 1 workers are
-           #       running CPLEX, and were started from the same directory.
-           #       these logs don't matter anyway (we redirect everything),
-           #       and are largely an annoyance.
-           try:
-              if  re.match('cplex\.log', filename) != None:
-                  os.remove(filename)
-              elif re.match('clone\d+\.log', filename) != None:
-                  os.remove(filename)
-           except OSError:
-              pass
+            # CPLEX temporary files come in two flavors - cplex.log and clone*.log.
+            # the latter is the case for multi-processor environments.
+            # IMPT: trap the possible exception raised by the file not existing.
+            #       this can occur in pyro environments where > 1 workers are
+            #       running CPLEX, and were started from the same directory.
+            #       these logs don't matter anyway (we redirect everything),
+            #       and are largely an annoyance.
+            try:
+                if  re.match('cplex\.log', filename) != None:
+                    os.remove(filename)
+                elif re.match('clone\d+\.log', filename) != None:
+                    os.remove(filename)
+            except OSError:
+                pass
 
         # let the base class deal with returning results.
         return ILMLicensedSystemCallSolver._postsolve(self)
 
 
-class MockCPLEX(CPLEX,mockmip.MockMIP):
+class MockCPLEX(CPLEXSHELL,mockmip.MockMIP):
     """A Mock CPLEX solver used for testing
     """
 
@@ -540,16 +570,16 @@ class MockCPLEX(CPLEX,mockmip.MockMIP):
 
     def __init__(self, **kwds):
         try:
-           CPLEX.__init__(self, **kwds)
+            CPLEXSHELL.__init__(self, **kwds)
         except pyutilib.common.ApplicationError: #pragma:nocover
-           pass                        #pragma:nocover
+            pass                        #pragma:nocover
         mockmip.MockMIP.__init__(self,"cplex")
 
     def available(self, exception_flag=True):
-        return CPLEX.available(self,exception_flag)
+        return CPLEXSHELL.available(self,exception_flag)
 
     def create_command_line(self,executable,problem_files):
-        command = CPLEX.create_command_line(self,executable,problem_files)
+        command = CPLEXSHELL.create_command_line(self,executable,problem_files)
         mockmip.MockMIP.create_command_line(self,executable,problem_files)
         return command
 
@@ -561,4 +591,4 @@ class MockCPLEX(CPLEX,mockmip.MockMIP):
 
 
 pyutilib.services.register_executable(name="cplex")
-
+pyutilib.services.register_executable(name="cplexamp")
