@@ -11,6 +11,7 @@
 
 import os
 import re
+import logging
 
 from coopr.opt.base import *
 from coopr.opt.results import *
@@ -23,10 +24,43 @@ from pyutilib.services import register_executable, registered_executable
 from pyutilib.services import TempfileManager
 
 
-class GLPK(SystemCallSolver):
-    """The GLPK LP/MIP solver"""
+class GLPK(OptSolver):
+    """The GLPK LP/MIP solver
+    """
 
-    alias('glpk', doc='Shell interface to the GNU Linear Programming Kit')
+    alias('glpk', doc='The GLPK MIP solver')
+
+    def __new__(cls, *args, **kwds):
+        try:
+            mode = kwds['solver_io']
+            if mode is None:
+                mode = 'lp'
+            del kwds['solver_io']
+        except KeyError:
+            mode = 'lp'
+        #
+        if mode  == 'lp':
+            return SolverFactory('_glpk_shell', **kwds)
+        if mode == 'python':
+            opt = SolverFactory('_glpk_direct', **kwds)
+            if opt is None:
+                logging.getLogger('coopr.plugins').error('Python API for GLPK is not installed')
+                return
+            return opt
+        #
+        if mode == 'os':
+            opt = SolverFactory('_ossolver', **kwds)
+        else:
+            logging.getLogger('coopr.plugins').error('Unknown IO type: %s' % mode)
+            return
+        opt.set_options('solver=glpsol')
+        return opt
+
+
+class GLPKSHELL(SystemCallSolver):
+    """Shell interface to the GLPK LP/MIP solver"""
+
+    alias('_glpk_shell', doc='Shell interface to the GNU Linear Programming Kit')
 
     def __init__(self, **kwds):
         #
@@ -285,6 +319,9 @@ class GLPK(SystemCallSolver):
                         elif len(tokens) == 3 and tokens[1] == "INTEGER" and \
                                  tokens[2] == "UNDEFINED":
                             soln.status = SolutionStatus.stoppedByLimit
+                        elif len(tokens) == 3 and tokens[1] == "INTEGER" and \
+                                tokens[2] == "EMPTY":
+                            soln.status = SolutionStatus.infeasible
                         elif (len(tokens) == 2) and (tokens[1] == "UNDEFINED"):
                             soln.status = SolutionStatus.infeasible
                         else:
@@ -517,9 +554,9 @@ class GLPK(SystemCallSolver):
                             else:
                                 raise ValueError, "Unknown status="+state_string+" encountered for variable="+variable_name+" in the following line of the GLPK solution file="+self.soln_file+":\n"+line
 
-                            soln.variable[variable_name].value = variable_value
+                            variable = soln.variable[variable_name] = {"Value" : variable_value, "Id" : len(soln.variable)}
                         else:
-                            soln.variable[variable_name].value = activity
+                            variable = soln.variable[variable_name] = {"Value" : activity, "Id" : len(soln.variable)}
 
                     # if all of the variables have been read, exit.
                     if number_of_variables_read == results.problem.number_of_variables:
@@ -560,7 +597,7 @@ class GLPK(SystemCallSolver):
                                eval(soln.objective(0))
 
 
-class MockGLPK(GLPK,mockmip.MockMIP):
+class MockGLPK(GLPKSHELL,mockmip.MockMIP):
     """A Mock GLPK solver used for testing
     """
 
@@ -568,16 +605,16 @@ class MockGLPK(GLPK,mockmip.MockMIP):
 
     def __init__(self, **kwds):
         try:
-           GLPK.__init__(self, **kwds)
+            GLPKSHELL.__init__(self, **kwds)
         except ApplicationError: #pragma:nocover
-           pass                        #pragma:nocover
+            pass                        #pragma:nocover
         mockmip.MockMIP.__init__(self,"glpk")
 
     def available(self, exception_flag=True):
-        return GLPK.available(self,exception_flag)
+        return GLPKSHELL.available(self,exception_flag)
 
     def create_command_line(self,executable,problem_files):
-        command = GLPK.create_command_line(self,executable,problem_files)
+        command = GLPKSHELL.create_command_line(self,executable,problem_files)
         mockmip.MockMIP.create_command_line(self,executable,problem_files)
         return command
 
@@ -589,9 +626,9 @@ class MockGLPK(GLPK,mockmip.MockMIP):
 
     def _convert_problem(self,args,pformat,valid_pformats):
         if pformat in [ProblemFormat.mps,ProblemFormat.cpxlp]:
-           return (args,pformat,None)
+            return (args,pformat,None)
         else:
-           return (args,ProblemFormat.cpxlp,None)
+            return (args,ProblemFormat.cpxlp,None)
 
 
 register_executable( name='glpsol')
