@@ -26,7 +26,7 @@ except ImportError:
 import pyutilib.misc
 import pyutilib.component.core
 from coopr.opt.parallel.manager import *
-from coopr.opt.parallel.solver import *
+from coopr.opt.parallel.async_solver import *
 from coopr.opt.results import SolverResults
 
 
@@ -41,6 +41,7 @@ class SolverManager_Pyro(AsynchronousSolverManager):
         AsynchronousSolverManager.clear(self)
         self.client = pyutilib.pyro.Client()
         self._opt = None
+        self._verbose = False
         self._ah = {}
 
     def _perform_queue(self, ah, *args, **kwds):
@@ -48,13 +49,17 @@ class SolverManager_Pyro(AsynchronousSolverManager):
         Perform the queue operation.  This method returns the ActionHandle,
         and the ActionHandle status indicates whether the queue was successful.
         """
-        
+
         if 'opt' in kwds:
             self._opt = kwds['opt']
             del kwds['opt']
         else:
             raise ActionManagerError, "No solver passed to SolverManager_Pyro, method=_perform_queue; use keyword option \"opt\""
 
+        if 'verbose' in kwds:
+            self._verbose = kwds['verbose']
+            del kwds['verbose']
+
         #
         # Force coopr.opt to ignore tests for availability, at least locally
         #
@@ -74,15 +79,15 @@ class SolverManager_Pyro(AsynchronousSolverManager):
         #
         solver_options = {}
         for key in self._opt.options:
-           solver_options[key]=self._opt.options[key]
+            solver_options[key]=self._opt.options[key]
 
         # pick up the warm-start file, if available.
         warm_start_file_string = None
         warm_start_file_name = None
         if hasattr(self._opt,  "warm_start_solve"):
-           if (self._opt.warm_start_solve is True) and (self._opt.warm_start_file_name is not None):
-              warm_start_file_name = self._opt.warm_start_file_name
-              warm_start_file_string = open(warm_start_file_name, 'r').read()
+            if (self._opt.warm_start_solve is True) and (self._opt.warm_start_file_name is not None):
+                warm_start_file_name = self._opt.warm_start_file_name
+                warm_start_file_string = open(warm_start_file_name, 'r').read()
 
         #
         # Pickl everything into one big data object via the "Bunch" command
@@ -93,9 +98,9 @@ class SolverManager_Pyro(AsynchronousSolverManager):
                                  warmstart_file=warm_start_file_string, warmstart_filename=warm_start_file_name, \
                                  kwds=kwds, solver_options=solver_options, suffixes=self._opt.suffixes, symbol_map=self._opt._symbol_map)
         task = pyutilib.pyro.Task(data=data, id=ah.id)
-        self.client.add_task(task)
+        self.client.add_task(task, self._verbose)
         self._ah[task.id] = ah
-        
+
         return ah
 
     def _perform_wait_any(self):
@@ -110,16 +115,15 @@ class SolverManager_Pyro(AsynchronousSolverManager):
             # this protects us against the case where we get an action
             # handle that we didn't know about or expect.
             while(True):
-               task = self.client.get_result()
-               if task.id in self._ah:
-                  ah = self._ah[task.id]
-                  self._ah[task.id] = None
-                  ah.status = ActionStatus.done
-                  #print "HERE",ah.id, task.result
-                  self.results[ah.id] = pickle.loads(task.result)
-                  #self.results[ah.id].write()
-                  return ah
+                task = self.client.get_result()
+                if task.id in self._ah:
+                    ah = self._ah[task.id]
+                    self._ah[task.id] = None
+                    ah.status = ActionStatus.done
+                    #print "HERE",ah.id, task.result
+                    self.results[ah.id] = pickle.loads(task.result)
+                    #self.results[ah.id].write()
+                    return ah
 
 if not using_pyro:
     SolverManagerFactory.deactivate('pyro')
-
