@@ -23,10 +23,13 @@ from coopr.opt import ResultsFormat, ProblemFormat, ConverterError
 pyutilib.services.register_executable('cplexamp')
 coopr.opt.SolverResults.default_print_options.ignore_time = True
 try:
+    cplex = coopr.plugins.mip.CPLEX(keepFiles=True)
+    cplex_available = (not cplex.executable() is None) and cplex.available(False)
     asl = coopr.plugins.mip.ASL(keepFiles=True, options={'solver':'cplexamp'})
-    cplexamp_available= (not asl.executable() is None) and asl.available(False)
+    cplexamp_available= cplex_available and (not asl.executable() is None) and asl.available(False)
 except pyutilib.common.ApplicationError:
     cplexamp_available=False
+
 try:
     pico_convert =  pyutilib.services.registered_executable("pico_convert")
     pico_convert_available= (not pico_convert is None)
@@ -63,7 +66,7 @@ class mock_all(unittest.TestCase):
     def test_path(self):
         """ Verify that the ASL path is what is expected """
         if type(self.asl) is 'ASL':
-           self.assertEqual(self.asl.executable.split(os.sep)[-1],"ASL"+coopr.util.executable_extension)
+            self.assertEqual(self.asl.executable.split(os.sep)[-1],"ASL"+coopr.util.executable_extension)
 
     def Xtest_solve1(self):
         """ Test ASL - test1.mps """
@@ -92,9 +95,9 @@ class mock_all(unittest.TestCase):
         results.write(filename=currdir+"test_solve3.txt",times=False)
         self.assertFileEqualsBaseline(currdir+"test_solve3.txt", currdir+"test2_asl.txt")
         if os.path.exists(currdir+"test2.solution.dat"):
-           os.remove(currdir+"test2.solution.dat")
+            os.remove(currdir+"test2.solution.dat")
         #os.remove(currdir+"test_solve3.log")
-        
+
     def test_solve4(self):
         """ Test ASL - test4.nl """
         if pico_convert_available:
@@ -124,36 +127,36 @@ class mock_all(unittest.TestCase):
     def Xtest_mock5(self):
         """ Mock Test ASL - test5.mps """
         if cplexamp_available:
-           pass
+            pass
         results = self.asl.solve(currdir+"test4.nl", logfile=currdir+"test_solve5.log", keepfiles=True)
         results.write(filename=currdir+"test_mock5.txt",times=False)
         self.assertFileEqualsBaseline(currdir+"test_mock5.txt", currdir+"test4_asl.txt")
         os.remove(currdir+"test4.sol")
         os.remove(currdir+"test_solve5.log")
-        
+
     def test_error1(self):
         """ Bad results format """
         try:
-          results = self.asl.solve(currdir+"test1.mps", format=ResultsFormat.sol)
-          self.fail("test_error1")
+            results = self.asl.solve(currdir+"test1.mps", format=ResultsFormat.sol)
+            self.fail("test_error1")
         except ValueError:
-          pass
+            pass
 
     def test_error2(self):
         """ Bad solve option """
         try:
-          results = self.asl.solve(currdir+"test1.mps", foo="bar")
-          self.fail("test_error2")
+            results = self.asl.solve(currdir+"test1.mps", foo="bar")
+            self.fail("test_error2")
         except ValueError:
-          pass
+            pass
 
     def test_error3(self):
         """ Bad solve option """
         try:
-          results = self.asl.solve(currdir+"bad.mps", foo="bar")
-          self.fail("test_error3")
+            results = self.asl.solve(currdir+"bad.mps", foo="bar")
+            self.fail("test_error3")
         except ValueError:
-          pass
+            pass
 
 
 class mip_all(mock_all):
@@ -165,4 +168,3 @@ mip_all = unittest.skipIf(not cplexamp_available, "The 'cplexamp' command is not
 
 if __name__ == "__main__":
     unittest.main()
-
